@@ -1,4 +1,4 @@
-/* $Id: openttd.cpp 24602 2012-10-17 18:49:42Z rubidium $ */
+/* $Id: openttd.cpp 26371 2014-02-23 22:03:08Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -61,11 +61,13 @@
 #include "game/game_config.hpp"
 #include "town.h"
 #include "subsidy_func.h"
+#include "gfx_layout.h"
+#include "viewport_sprite_sorter.h"
 
+#include "linkgraph/linkgraphschedule.h"
 
 #include <stdarg.h>
 
-
 void CallLandscapeTick();
 void IncreaseDate();
 void DoPaletteAnimations();
@@ -74,7 +76,6 @@ void ResetMusic();
 void CallWindowTickEvent();
 bool HandleBootstrap();
 
-extern void SetDifficultyLevel(int mode, DifficultySettings *gm_opt);
 extern Company *DoStartupNewCompany(bool is_ai, CompanyID company = INVALID_COMPANY);
 extern void ShowOSErrorBox(const char *buf, bool system);
 extern char *_config_file;
@@ -188,10 +189,13 @@ static void ShowHelp()
 	p = BaseMusic::GetSetsList(p, lastof(buf));
 
 	/* List the drivers */
-	p = VideoDriverFactoryBase::GetDriversInfo(p, lastof(buf));
+	p = DriverFactoryBase::GetDriversInfo(p, lastof(buf));
 
 	/* List the blitters */
-	p = BlitterFactoryBase::GetBlittersInfo(p, lastof(buf));
+	p = BlitterFactory::GetBlittersInfo(p, lastof(buf));
+
+	/* List the debug facilities. */
+	p = DumpDebugFacilityNames(p, lastof(buf));
 
 	/* We need to initialize the AI, so it finds the AIs */
 	AI::Initialize();
@@ -295,6 +299,7 @@ static void ShutdownGame()
 	free(_config_file);
 #endif
 
+	LinkGraphSchedule::Clear();
 	PoolBase::Clean(PT_ALL);
 
 	/* No NewGRFs were loaded when it was still bootstrapping. */
@@ -423,6 +428,7 @@ struct AfterNewGRFScan : NewGRFScanCallback {
 		CheckConfig();
 		LoadFromHighScore();
 		LoadHotkeysFromConfig();
+		WindowDesc::LoadFromConfig();
 
 		/* We have loaded the config, so we may possibly save it. */
 		*save_config_ptr = save_config;
@@ -518,8 +524,13 @@ static const OptionData _options[] = {
 	GETOPT_END()
 };
 
-
-int ttd_main(int argc, char *argv[])
+/**
+ * Main entry point for this lovely game.
+ * @param argc The number of arguments passed to this game.
+ * @param argv The values of the arguments.
+ * @return 0 when there is no error.
+ */
+int openttd_main(int argc, char *argv[])
 {
 	char *musicdriver = NULL;
 	char *sounddriver = NULL;
@@ -545,6 +556,7 @@ int ttd_main(int argc, char *argv[])
 	_config_file = NULL;
 
 	GetOptData mgo(argc - 1, argv + 1, _options);
+	int ret = 0;
 
 	int i;
 	while ((i = mgo.GetOpt()) != -1) {
@@ -626,7 +638,11 @@ int ttd_main(int argc, char *argv[])
 			break;
 		case 'q': {
 			DeterminePaths(argv[0]);
-			if (StrEmpty(mgo.opt)) return 1;
+			if (StrEmpty(mgo.opt)) {
+				ret = 1;
+				goto exit_noshutdown;
+			}
+
 			char title[80];
 			title[0] = '\0';
 			FiosGetSavegameListCallback(SLD_LOAD_GAME, mgo.opt, strrchr(mgo.opt, '.'), title, lastof(title));
@@ -641,12 +657,12 @@ int ttd_main(int argc, char *argv[])
 					GetString(buf, _load_check_data.error, lastof(buf));
 					fprintf(stderr, "%s\n", buf);
 				}
-				return 1;
+				goto exit_noshutdown;
 			}
 
 			WriteSavegameInfo(title);
 
-			return 0;
+			goto exit_noshutdown;
 		}
 		case 'G': scanner->generation_seed = atoi(mgo.opt); break;
 		case 'c': _config_file = strdup(mgo.opt); break;
@@ -670,8 +686,8 @@ int ttd_main(int argc, char *argv[])
 		BaseSounds::FindSets();
 		BaseMusic::FindSets();
 		ShowHelp();
-		delete scanner;
-		return 0;
+
+		goto exit_noshutdown;
 	}
 
 #if defined(WINCE) && defined(_DEBUG)
@@ -740,8 +756,8 @@ int ttd_main(int argc, char *argv[])
 	if (blitter == NULL && _ini_blitter != NULL) blitter = strdup(_ini_blitter);
 	_blitter_autodetected = StrEmpty(blitter);
 	/* If we have a 32 bpp base set, try to select the 32 bpp blitter first, but only if we autoprobe the blitter. */
-	if (!_blitter_autodetected || BaseGraphics::GetUsedSet() == NULL || BaseGraphics::GetUsedSet()->blitter == BLT_8BPP || BlitterFactoryBase::SelectBlitter("32bpp-anim") == NULL) {
-		if (BlitterFactoryBase::SelectBlitter(blitter) == NULL) {
+	if (!_blitter_autodetected || BaseGraphics::GetUsedSet() == NULL || BaseGraphics::GetUsedSet()->blitter == BLT_8BPP || BlitterFactory::SelectBlitter("32bpp-anim") == NULL) {
+		if (BlitterFactory::SelectBlitter(blitter) == NULL) {
 			StrEmpty(blitter) ?
 				usererror("Failed to autoprobe blitter") :
 				usererror("Failed to select requested blitter '%s'; does it exist?", blitter);
@@ -750,7 +766,7 @@ int ttd_main(int argc, char *argv[])
 	free(blitter);
 
 	if (videodriver == NULL && _ini_videodriver != NULL) videodriver = strdup(_ini_videodriver);
-	_video_driver = (VideoDriver*)VideoDriverFactoryBase::SelectDriver(videodriver, Driver::DT_VIDEO);
+	_video_driver = (VideoDriver*)DriverFactoryBase::SelectDriver(videodriver, Driver::DT_VIDEO);
 	if (_video_driver == NULL) {
 		StrEmpty(videodriver) ?
 			usererror("Failed to autoprobe video driver") :
@@ -758,6 +774,8 @@ int ttd_main(int argc, char *argv[])
 	}
 	free(videodriver);
 
+	InitializeSpriteSorter();
+
 	/* Initialize the zoom level of the screen to normal */
 	_screen.zoom = ZOOM_LVL_NORMAL;
 
@@ -778,7 +796,11 @@ int ttd_main(int argc, char *argv[])
 	}
 #endif /* ENABLE_NETWORK */
 
-	if (!HandleBootstrap()) goto exit;
+	if (!HandleBootstrap()) {
+		ShutdownGame();
+
+		goto exit_bootstrap;
+	}
 
 	_video_driver->ClaimMousePointer();
 
@@ -812,7 +834,7 @@ int ttd_main(int argc, char *argv[])
 	free(music_set);
 
 	if (sounddriver == NULL && _ini_sounddriver != NULL) sounddriver = strdup(_ini_sounddriver);
-	_sound_driver = (SoundDriver*)SoundDriverFactoryBase::SelectDriver(sounddriver, Driver::DT_SOUND);
+	_sound_driver = (SoundDriver*)DriverFactoryBase::SelectDriver(sounddriver, Driver::DT_SOUND);
 	if (_sound_driver == NULL) {
 		StrEmpty(sounddriver) ?
 			usererror("Failed to autoprobe sound driver") :
@@ -821,7 +843,7 @@ int ttd_main(int argc, char *argv[])
 	free(sounddriver);
 
 	if (musicdriver == NULL && _ini_musicdriver != NULL) musicdriver = strdup(_ini_musicdriver);
-	_music_driver = (MusicDriver*)MusicDriverFactoryBase::SelectDriver(musicdriver, Driver::DT_MUSIC);
+	_music_driver = (MusicDriver*)DriverFactoryBase::SelectDriver(musicdriver, Driver::DT_MUSIC);
 	if (_music_driver == NULL) {
 		StrEmpty(musicdriver) ?
 			usererror("Failed to autoprobe music driver") :
@@ -840,7 +862,9 @@ int ttd_main(int argc, char *argv[])
 
 	CheckForMissingGlyphs();
 
+	/* ScanNewGRFFiles now has control over the scanner. */
 	ScanNewGRFFiles(scanner);
+	scanner = NULL;
 
 	_video_driver->MainLoop();
 
@@ -850,22 +874,47 @@ int ttd_main(int argc, char *argv[])
 	if (save_config) {
 		SaveToConfig();
 		SaveHotkeysToConfig();
+		WindowDesc::SaveToConfig();
 		SaveToHighScore();
 	}
 
-exit:
 	/* Reset windowing system, stop drivers, free used memory, ... */
 	ShutdownGame();
+	goto exit_normal;
 
+exit_noshutdown:
+	/* These three are normally freed before bootstrap. */
+	free(graphics_set);
+	free(videodriver);
+	free(blitter);
+
+exit_bootstrap:
+	/* These are normally freed before exit, but after bootstrap. */
+	free(sounds_set);
+	free(music_set);
+	free(musicdriver);
+	free(sounddriver);
+
+exit_normal:
 	free(BaseGraphics::ini_set);
 	free(BaseSounds::ini_set);
 	free(BaseMusic::ini_set);
+
 	free(_ini_musicdriver);
 	free(_ini_sounddriver);
 	free(_ini_videodriver);
 	free(_ini_blitter);
 
-	return 0;
+	delete scanner;
+
+#ifdef ENABLE_NETWORK
+	extern FILE *_log_fd;
+	if (_log_fd != NULL) {
+		fclose(_log_fd);
+	}
+#endif /* ENABLE_NETWORK */
+
+	return ret;
 }
 
 void HandleExitGameRequest()
@@ -887,6 +936,7 @@ static void MakeNewGameDone()
 	/* In a dedicated server, the server does not play */
 	if (!_video_driver->HasGUI()) {
 		SetLocalCompany(COMPANY_SPECTATOR);
+		if (_settings_client.gui.pause_on_newgame) DoCommandP(0, PM_PAUSED_NORMAL, 1, CMD_PAUSE);
 		IConsoleCmdExec("exec scripts/game_start.scr 0");
 		return;
 	}
@@ -913,6 +963,7 @@ static void MakeNewGameDone()
 
 	if (_settings_client.gui.pause_on_newgame) DoCommandP(0, PM_PAUSED_NORMAL, 1, CMD_PAUSE);
 
+	CheckEngines();
 	MarkWholeScreenDirty();
 }
 
@@ -1025,7 +1076,7 @@ void SwitchToMode(SwitchMode new_mode)
 		}
 	}
 #endif /* ENABLE_NETWORK */
-	/* Make sure all AI controllers are gone at quiting game */
+	/* Make sure all AI controllers are gone at quitting game */
 	if (new_mode != SM_SAVE_GAME) AI::KillAll();
 
 	switch (new_mode) {
@@ -1052,7 +1103,8 @@ void SwitchToMode(SwitchMode new_mode)
 				ShowErrorMessage(STR_JUST_RAW_STRING, INVALID_STRING_ID, WL_ERROR);
 			} else {
 				if (_saveload_mode == SLD_LOAD_SCENARIO) {
-					StartupEngines();
+					/* Reset engine pool to simplify changing engine NewGRFs in scenario editor. */
+					EngineOverrideManager::ResetToCurrentNewGRFConfig();
 				}
 				/* Update the local company for a loaded game. It is either always
 				 * company #1 (eg 0) or in the case of a dedicated server a spectator */
@@ -1224,7 +1276,7 @@ static void CheckCaches()
 		}
 
 		switch (v->type) {
-			case VEH_TRAIN:    Train::From(v)->ConsistChanged(true);     break;
+			case VEH_TRAIN:    Train::From(v)->ConsistChanged(CCF_TRACK); break;
 			case VEH_ROAD:     RoadVehUpdateCache(RoadVehicle::From(v)); break;
 			case VEH_AIRCRAFT: UpdateAircraftCache(Aircraft::From(v));   break;
 			case VEH_SHIP:     Ship::From(v)->UpdateCache();             break;
@@ -1292,22 +1344,25 @@ static void CheckCaches()
  */
 void StateGameLoop()
 {
-	/* dont execute the state loop during pause */
+	/* don't execute the state loop during pause */
 	if (_pause_mode != PM_UNPAUSED) {
 		UpdateLandscapingLimits();
+#ifndef DEBUG_DUMP_COMMANDS
 		Game::GameLoop();
+#endif
 		CallWindowTickEvent();
 		return;
 	}
 	if (HasModalProgress()) return;
 
-	ClearStorageChanges(false);
+	Layouter::ReduceLineCache();
 
 	if (_game_mode == GM_EDITOR) {
+		BasePersistentStorageArray::SwitchMode(PSM_ENTER_GAMELOOP);
 		RunTileLoop();
 		CallVehicleTicks();
 		CallLandscapeTick();
-		ClearStorageChanges(true);
+		BasePersistentStorageArray::SwitchMode(PSM_LEAVE_GAMELOOP);
 		UpdateLandscapingLimits();
 
 		CallWindowTickEvent();
@@ -1326,15 +1381,18 @@ void StateGameLoop()
 		 *  for multiplayer compatibility */
 		Backup<CompanyByte> cur_company(_current_company, OWNER_NONE, FILE_LINE);
 
+		BasePersistentStorageArray::SwitchMode(PSM_ENTER_GAMELOOP);
 		AnimateAnimatedTiles();
 		IncreaseDate();
 		RunTileLoop();
 		CallVehicleTicks();
 		CallLandscapeTick();
-		ClearStorageChanges(true);
+		BasePersistentStorageArray::SwitchMode(PSM_LEAVE_GAMELOOP);
 
+#ifndef DEBUG_DUMP_COMMANDS
 		AI::GameLoop();
 		Game::GameLoop();
+#endif
 		UpdateLandscapingLimits();
 
 		CallWindowTickEvent();
@@ -1391,8 +1449,8 @@ void GameLoop()
 
 	/* autosave game? */
 	if (_do_autosave) {
-		_do_autosave = false;
 		DoAutosave();
+		_do_autosave = false;
 		SetWindowDirty(WC_STATUS_BAR, 0);
 	}
 
